import os
import re
import yaml
import json
import traceback



# Every file matching the pattern gets thrown into a Page object
class Page():
    def __init__(self, fileName, input_dir, root, data_format):
        self.file_name, self.fileExtension = os.path.splitext(fileName)
        self.file_path = os.path.join(root, fileName)
        self.relative_path = self.file_path.replace(input_dir+"/", "")        
        self.output_filename = ""
        self.output_path = ""
        self.mtime = self.get_modified_time()
        self.data_format = data_format
        self.file_contents = self.read_file(self.file_path)
        self.data = self.get_data(self.file_contents)
        self.meta = self.get_metadata(self.data)
        self.content = self.get_content(self.data)

    def read_file(self, file_path):
        source_file = open(file_path, 'r')
        file_contents = ""
        try:
            file_contents = source_file.read().decode('utf8')
        except:
            print "\033[31mTrouble reading %s\033[0m" % (source_file)
            traceback.print_exc() 
        
        return file_contents

    def get_data(self, file_contents):
        data = {
            "meta":{
                "title": None,
                "template": None,
                "description": None
            }
        }

        if self.data_format == "json":
            data.update(self.get_as_json(file_contents))

        if self.data_format == "yaml":
            data.update(self.get_as_yaml(file_contents))

        return data

    def get_as_json(self, file_contents):
        json_data = {}
        try:
            json_data = json.loads(file_contents)
        except:
            print "\033[31mCouldn't decode %s as JSON\033[0m" % (self.file_path)
            traceback.print_exc() 

        return json_data

    def get_as_yaml(self, file_contents):
        yaml_data = {}
        try:
            yaml_data = yaml.load(file_contents)
        except:
            print "\033[31mCouldn't decode %s as YAML\033[0m" % (self.file_path)
            traceback.print_exc() 
            
        return yaml_data

    def get_metadata(self, data):
        if "meta" in data:
            return data["meta"]
        return {title: "None"}

    def get_content(self, data):
        content = data.copy()
        content.pop("meta", None)
        return content

    def get_page_content(self):
        if "data" in self.content:
            return self.content["data"]
        return None

    def set_page_content(self, str):
        self.file_contents = str

    def get_page_vars(self):
        self.meta["slug"] = self.output_filename
        return self.meta

    def get_modified_time(self):
        mtime = os.path.getmtime(self.file_path)
        return mtime
        

    def get_template(self):
        template = 0

        if self.meta:
            if self.meta['template']:
                template = self.meta['template']
        
        return template


    def set_output_path(self, output_dir, output_file_extension):
        name = self.relative_path.replace(self.fileExtension, ".%s" % (output_file_extension), 1)
        path = "%s/%s" % (output_dir, name)
        self.output_path = re.sub("/{2,}", "/", path)
        self.output_filename = name

    def get_output_path(self):
        return self.output_path

    def get_file_path(self):
        return self.file_path

    def get_relative_path(self):
        return self.relative_path



import os
import subprocess
import shelve 
import sys 

# Takes a directory and transforms the matching elements into the matching class.
class DirectoryWalker():
    def __init__(self, params):
        self.item_class = params['item_class']
        self.input_dir = params['input_dir']
        self.ignore_files = params['ignore_files']
        self.data_format = params['data_format']
        self.nocache = params['nocache']

    def fetch(self):
        items = []

        # Not supported in python 2.6 hmmm
        # if sys.version_info > (2,7,0):
            # files = subprocess.check_output(["find", self.input_dir, "-name", "*.%s" % (self.data_format)], stderr=subprocess.STDOUT)
        # else:
        p = subprocess.Popen(["find", self.input_dir, "-name", "*.%s" % (self.data_format)], stdout=subprocess.PIPE)

        out, err = p.communicate()
        files = out.split("\n")

        shelf = shelve.open("wrangler_cache.db") 

        for f in files: 
            if f != '':

                # If the key doesn't exist on the shelf, or the mtime is different, re-cache the object
                if (not shelf.has_key(f)) or (shelf[f].mtime < os.path.getmtime(f)) or (self.nocache):
                    item = self.item_class(f, self.input_dir,  "", self.data_format)
                    shelf[f] = item

                    print "\033[1;35mCaching \033[0m\033[2m%s\033[0m" % (f)

                items.append(shelf[f])

        try:
            shelf.sync()
        finally:
            shelf.close()

        return items